import logging

from Function import Function

# https://github.com/intel/linux-sgx/blob/master/SampleCode/SampleEnclave/Enclave/Edger8rSyntax/Pointers.edl

class EdlFileWriter:
    
    def __init__ (self, filePath):
        self.trustedFunctions = {}
        self.untrustedFunctions = {}
        self.types = [] #list of code representing struct union and enum
        self.filePath = filePath
        self.define = ""

    def addDefines(self, define: str):
        "Add #define at the start of the edl file"
        self.define = define


    def addTrustedFunction(self, function: Function):
        """ Add a new trusted function if it is not already present in untrusted section """
        if function.functionName in self.untrustedFunctions:  
            raise Exception("Error #E001: the function trusted" + function.functionName + "is already present in untrusted function")
        
        self.trustedFunctions[function.functionName] = function

    def addUntrustedFunction(self, function: Function):
        """ Add a new Untrusted function if it is not already present in trusted section """
        if function.functionName in self.trustedFunctions:  
            raise Exception("Error #E002: the function untrusted" + function.functionName + "is already present in trusted function")
        
        self.untrustedFunctions[function.functionName] = function

        
    def setTypeDefinitions(self, typeDefinitionCode: []):
        self.types = typeDefinitionCode 

    def writeEdlFile(self):
        """ Try to create a new file, override the file if already exists """
        with open(self.filePath, "w") as file: 
            logging.info("Writing file: " + self.filePath )
            file.write(self.getContent())
            
    def getContent(self): 
        fileContent =  f"""
/*
* This code as been generated automatically by Python Intel SGX converter script
*/

{self.define}

enclave {{

"""
        for typedef in self.types:
            fileContent +=  "        " + typedef.replace("\n", "\n        ")  + "\n\n"
        
        fileContent += """
        trusted {
"""
        for f in self.trustedFunctions:
            trusted: Function = self.trustedFunctions[f]
            fileContent += "            public " + trusted.returnType + " " + trusted.functionName + trusted.getSgxForEdl() + ';\n'
        
        fileContent += """
        };

        untrusted {        
"""
        for f in self.untrustedFunctions:
            untrusted: Function = self.untrustedFunctions[f]
            fileContent += "             " + untrusted.returnType + " " + untrusted.functionName + untrusted.getSgxForEdl() + ';\n'

        fileContent += """
        };
};
"""
        return fileContent



